@@ -1,4 +1,4 @@
-use crate::{Rule, RuleWithConstraint};
+use crate::{Rule, RuleCore};
 
 use ast_grep_core::language::Language;
 use ast_grep_core::meta_var::MetaVarEnv;
@@ -20,17 +20,18 @@ impl<R> Clone for Registration<R> {
 }
 
 impl<R> Registration<R> {
-  fn read(&self) -> RwLockReadGuard<HashMap<String, R>> {
+  // TODO: this is sooo wrong
+  pub(crate) fn read(&self) -> RwLockReadGuard<HashMap<String, R>> {
     self.0.read().unwrap()
   }
   fn write(&self) -> RwLockWriteGuard<HashMap<String, R>> {
     self.0.write().unwrap()
   }
 }
-pub type GlobalRules<L> = Registration<RuleWithConstraint<L>>;
+pub type GlobalRules<L> = Registration<RuleCore<L>>;
 
 impl<L: Language> GlobalRules<L> {
-  pub fn insert(&self, id: &str, rule: RuleWithConstraint<L>) -> Result<(), ReferentRuleError> {
+  pub fn insert(&self, id: &str, rule: RuleCore<L>) -> Result<(), ReferentRuleError> {
     let mut map = self.write();
     if map.contains_key(id) {
       return Err(ReferentRuleError::DuplicateRule(id.into()));
@@ -54,8 +55,12 @@ impl<R> Default for Registration<R> {
 
 #[derive(Clone)]
 pub struct RuleRegistration<L: Language> {
+  /// utility rule to every RuleCore, every sub-rule has its own local utility
   local: Registration<Rule<L>>,
-  global: Registration<RuleWithConstraint<L>>,
+  /// global rules are shared by all RuleConfigs. It is a singleton.
+  global: Registration<RuleCore<L>>,
+  /// Every RuleConfig has its own rewriters. But sub-rules share parent's rewriters.
+  rewriters: Registration<RuleCore<L>>,
 }
 
 // these are shit code
@@ -64,14 +69,27 @@ impl<L: Language> RuleRegistration<L> {
     self.local.read()
   }
 
-  fn get_global(&self) -> RwLockReadGuard<HashMap<String, RuleWithConstraint<L>>> {
+  fn get_global(&self) -> RwLockReadGuard<HashMap<String, RuleCore<L>>> {
     self.global.read()
   }
 
+  pub fn get_rewriters(&self) -> GlobalRules<L> {
+    self.rewriters.clone()
+  }
+
   pub fn from_globals(global: &GlobalRules<L>) -> Self {
     Self {
       local: Default::default(),
       global: global.clone(),
+      rewriters: Default::default(),
+    }
+  }
+
+  pub fn with_rewriters(&self, rewriters: &GlobalRules<L>) -> Self {
+    Self {
+      local: self.local.clone(),
+      global: self.global.clone(),
+      rewriters: rewriters.clone(),
     }
   }
 
@@ -101,20 +119,25 @@ impl<L: Language> Default for RuleRegistration<L> {
     Self {
       local: Default::default(),
       global: Default::default(),
+      rewriters: Default::default(),
     }
   }
 }
 
 pub struct RegistrationRef<L: Language> {
   local: Weak<RwLock<HashMap<String, Rule<L>>>>,
-  global: Weak<RwLock<HashMap<String, RuleWithConstraint<L>>>>,
+  global: Weak<RwLock<HashMap<String, RuleCore<L>>>>,
 }
 // these are shit code
 impl<L: Language> RegistrationRef<L> {
   pub fn unref(&self) -> RuleRegistration<L> {
     let local = Registration(self.local.upgrade().unwrap());
     let global = Registration(self.global.upgrade().unwrap());
-    RuleRegistration { local, global }
+    RuleRegistration {
+      local,
+      global,
+      rewriters: Default::default(),
+    }
   }
 }
 
@@ -156,7 +179,7 @@ impl<L: Language> ReferentRule<L> {
 
   fn eval_global<F, T>(&self, func: F) -> Option<T>
   where
-    F: FnOnce(&RuleWithConstraint<L>) -> T,
+    F: FnOnce(&RuleCore<L>) -> T,
   {
     let registration = self.reg_ref.unref();
     let rules = registration.get_global();
